"""(c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017"""
import os

from epflldap.settings import CONSTANTS


def get_optional_env(key):
    """
    Return the value of an optional environment variable, and use
    the provided default if it's not set.
    """
    environment_variable_value = os.environ.get(key)
    if environment_variable_value:
        return environment_variable_value
    elif key in CONSTANTS:
        return CONSTANTS[key]
    else:
        raise Exception("The variable {1} is not set".format(key))



"""(c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017"""
import re
import ldap3

from epflldap.utils import get_optional_env


class Authenticator:
    """ Class to authenticate users using LDAPS """

    def __init__(self):
        self.ldap_server = get_optional_env('EPFL_LDAP_SERVER_FOR_AUTHENTICATE')
        self.protocol = 'ldaps' if get_optional_env('EPFL_LDAP_USE_SSL') == 'true' else 'ldap'

        self.use_ssl = True if get_optional_env('EPFL_LDAP_USE_SSL') == 'true' else False
        self.uri = self.protocol + '://' + self.ldap_server
        self.dn = get_optional_env('EPFL_LDAP_BASE_DN')
        self.user_attr = get_optional_env('EPFL_LDAP_USER_SEARCH_ATTR')

    def get_user_dn(self, username):
        """
        Get the user distinguished name (dn)
        """

        server = ldap3.Server('ldap://' + self.ldap_server)
        connection = ldap3.Connection(server)
        connection.open()

        connection.search(
            search_base=self.dn,
            search_filter='(' + self.user_attr + '=' + username + ')'
        )
        return connection.response[0]['dn']

    def authenticate(self, username, password):
        """
        Authenticate the user with a bind on the LDAP server
        """

        if username is None or password is None:
            return False

        # check the username
        if not re.match("^[A-Za-z0-9_-]*$", username):
            return False

        user_dn = self.get_user_dn(username)

        server = ldap3.Server(
                self.uri,
                use_ssl=self.use_ssl
            )

        connection = ldap3.Connection(server, user=user_dn, password=password)

        return connection.bind()



"""(c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017"""
from epflldap.ldap_authenticate import Authenticator
from epflldap.utils import get_optional_env


def test_get_user_dn():
    auth = Authenticator()
    user_dn = auth.get_user_dn(username='kermit')
    assert user_dn == "uid=kermit,ou=users,o=epfl,c=ch"


def test_authenticate():

    # Success test
    auth = Authenticator()
    username = get_optional_env('EPFL_LDAP_TEST_CORRECT_USERNAME')
    password = get_optional_env('EPFL_LDAP_TEST_CORRECT_PWD')
    assert auth.authenticate(username=username, password=password)

    # Failed test
    auth = Authenticator()
    username = 'kermit'
    password = 'badpassword'
    assert not auth.authenticate(username=username, password=password)



"""(c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017"""
from epflldap.ldap_search import get_username, get_sciper, get_units, is_unit_exist, get_unit_name, get_email


def test_get_sciper():
    assert get_sciper(username="charmier") == "188475"
    assert get_sciper(username='kermit') == "133134"


def test_get_username():
    assert get_username(sciper="188475") == "charmier"
    assert get_username(sciper="133134") == "kermit"


def test_get_email():
    assert get_email(sciper="133134") == "kermit.lagrenouille@epfl.ch"
    assert get_email(sciper="188475") == "gregory.charmier@epfl.ch"


def test_get_units():
    units = get_units(username="charmier")
    assert len(units) == 1
    assert '13030' == units[0]

    units = get_units(username="ebreton")
    assert len(units) == 3
    assert '13029' == units[0]
    assert '13030' == units[1]
    assert '13051' == units[2]


def test_is_unit_exist():
    # Success
    assert is_unit_exist(unit_id="13030")
    # Fail
    assert not is_unit_exist(unit_id="88")


def test_get_unit_name():
    unit = get_unit_name(unit_id="13030")
    assert unit.lower() == "idevelop"



"""(c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017"""
CONSTANTS = {
    "EPFL_LDAP_SERVER_FOR_SEARCH": "ldap.epfl.ch",
    "EPFL_LDAP_SERVER_FOR_AUTHENTICATE": "scoldap.epfl.ch",
    "EPFL_LDAP_BASE_DN": "o=epfl,c=ch",
    "EPFL_LDAP_USE_SSL": True,
    "EPFL_LDAP_USER_SEARCH_ATTR": "uid",
    "EPFL_LDAP_TEST_CORRECT_USERNAME": "kermit",
}



"""(c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017"""
try:
    from unittest.mock import MagicMock, patch
except:
    from mock import patch, MagicMock


@patch('epflldap.ldap_search.get_sciper')
def test_get_sciper(get_sciper):

    mock_get_sciper = MagicMock(return_value="188475")
    get_sciper.return_value = mock_get_sciper()
    assert get_sciper(username="charmier") == "188475"

    mock_get_sciper = MagicMock(return_value="133134")
    get_sciper.return_value = mock_get_sciper()
    assert get_sciper(username='kermit') == "133134"


@patch('epflldap.ldap_search.get_username')
def test_get_username(get_username):

    mock_get_username = MagicMock(return_value="charmier")
    get_username.return_value = mock_get_username()
    assert get_username(sciper="188475") == "charmier"

    mock_get_username = MagicMock(return_value="kermit")
    get_username.return_value = mock_get_username()
    assert get_username(sciper="133134") == "kermit"


@patch('epflldap.ldap_search.get_email')
def test_get_email(get_email):

    mock_get_email = MagicMock(return_value="kermit.lagrenouille@epfl.ch")
    get_email.return_value = mock_get_email()
    assert get_email(sciper="133134") == "kermit.lagrenouille@epfl.ch"

    mock_get_email = MagicMock(return_value="gregory.charmier@epfl.ch")
    get_email.return_value = mock_get_email()
    assert get_email(sciper="188475") == "gregory.charmier@epfl.ch"


@patch('epflldap.ldap_search.get_units')
def test_get_units(get_units):

    mock_get_units = MagicMock(return_value=["13030"])
    get_units.return_value = mock_get_units()

    units = get_units(username="charmier")
    assert len(units) == 1
    assert '13030' == units[0]

    mock_get_units = MagicMock(return_value=["13029", "13030", "13051"])
    get_units.return_value = mock_get_units()

    units = get_units(username="ebreton")
    assert len(units) == 3
    assert '13029' == units[0]
    assert '13030' == units[1]
    assert '13051' == units[2]


@patch('epflldap.ldap_search.is_unit_exist')
def test_is_unit_exist(is_unit_exist):

    mock_is_unit_exist = MagicMock(return_value=True)
    is_unit_exist.return_value = mock_is_unit_exist()

    # Success
    assert is_unit_exist(unit_id="13030")

    mock_is_unit_exist = MagicMock(return_value=False)
    is_unit_exist.return_value = mock_is_unit_exist()

    # Fail
    assert not is_unit_exist(unit_id="88")


@patch('epflldap.ldap_search.get_unit_name')
def test_get_unit_name(get_unit_name):

    mock_get_unit_name = MagicMock(return_value="idevelop")
    get_unit_name.return_value = mock_get_unit_name()

    unit = get_unit_name(unit_id="13030")
    assert unit.lower() == "idevelop"


@patch('epflldap.ldap_authenticate.Authenticator.authenticate')
def test_authenticate(authenticate):

    mock_authenticate = MagicMock(return_value=True)
    authenticate.return_value = mock_authenticate()

    # Success test
    username = "kermit"
    password = "rightpassword"
    assert authenticate(username=username, password=password)

    mock_authenticate = MagicMock(return_value=False)
    authenticate.return_value = mock_authenticate()

    # Success test
    username = "kermit"
    password = "badpassword"
    assert not authenticate(username=username, password=password)



"""(c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017"""
import ldap3

from epflldap.utils import get_optional_env


def _get_LDAP_connection():
    """
    Return a LDAP connection
    """
    server = ldap3.Server('ldap://' + get_optional_env('EPFL_LDAP_SERVER_FOR_SEARCH'))
    connection = ldap3.Connection(server)
    connection.open()

    return connection, get_optional_env('EPFL_LDAP_BASE_DN')


def LDAP_search(pattern_search, attribute):
    """
    Do a LDAP search
    """
    connection, ldap_base = _get_LDAP_connection()

    connection.search(
        search_base=ldap_base,
        search_filter=pattern_search,
        attributes=[attribute]
    )
    return connection.response


def is_unit_exist(unit_id):
    """
    Return True if the unit 'unid_id' exists.
    Otherwise return False
    """
    attribute = 'objectClass'
    response = LDAP_search(
        pattern_search="(uniqueidentifier=" + unit_id + ")",
        attribute=attribute
    )

    try:
        return 'EPFLorganizationalUnit' in response[0]['attributes'][attribute]
    except Exception:
        return False


def get_unit_name(unit_id):
    """
    Return the unit name to the unit 'unit_id'
    """
    attribute = 'cn'
    response = LDAP_search(
        pattern_search='(uniqueIdentifier=' + unit_id + ')',
        attribute=attribute
    )
    return response[0]['attributes'][attribute][0]


def get_units(username):
    """
    Return all units of user 'username'
    """
    connection, ldap_base = _get_LDAP_connection()

    # Search the user dn
    connection.search(
        search_base=ldap_base,
        search_filter='(uid=' + username + '@*)',
    )

    # For each user dn give me the unit
    dn_list = [connection.response[index]['dn'] for index in range(len(connection.response))]

    units = []
    # For each unit search unit information and give me the unit id
    for dn in dn_list:
        unit = dn.split(",ou=")[1]
        connection.search(search_base=ldap_base, search_filter='(ou=' + unit + ')', attributes=['uniqueidentifier'])
        units.append(connection.response[0]['attributes']['uniqueIdentifier'][0])

    return units


def get_sciper(username):
    """
    Return the sciper of user
    """
    attribute = 'uniqueIdentifier'
    response = LDAP_search(
        pattern_search='(uid=' + username + ')',
        attribute=attribute
    )
    return response[0]['attributes'][attribute][0]


def get_username(sciper):
    """
    Return username of user
    """
    attribute = 'uid'
    response = LDAP_search(
        pattern_search='(uniqueIdentifier=' + sciper + ')',
        attribute=attribute
    )
    return response[0]['attributes'][attribute][0]


def get_email(sciper):
    """
    Return username of user
    """
    attribute = 'mail'
    response = LDAP_search(
        pattern_search='(uniqueIdentifier=' + sciper + ')',
        attribute=attribute
    )
    return response[0]['attributes'][attribute][0]



# coding:utf-8
from setuptools import find_packages, setup
import epflldap


try:
    # Get the long description from the README file
    with open('README.md') as f:
        long_description = f.read()
except:
    long_description = "See README file"

setup(
    name='epfl-ldap',

    version=epflldap.__version__,

    description='A simple package to interact with the EPFL LDAP',
    long_description=long_description,

    url='https://github.com/epfl-idevelop/epfl-ldap',

    author='Charmier Grégory',
    author_email='gregory.charmier@epfl.ch',

    license="MIT",

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='EPFL development LDAP search authenticate',

    packages=find_packages(),

    install_requires=["ldap==1.0.2", "mock==2.0.0"],
)
